/*
 * Remembers which button in the planning table was clicked.
 */
var clickedButton;

/**
 * Initialize the widget. Called once.
 * 
 * This function implements the PageLoad event handler,
 * initialized the widget and
 * registers the button click event handler. 
 * 
 * @returns void
 */
function initializeWidget()
{
	/*
	 * Pad a number to 2 positions with leading zero.
	 * 
	 * @param integer number
	 *   Number to pad.
	 * @returns string|integer
	 */
    function pad(number) {
    	if (number < 10) {
    		return '0' + number;
    	}
    	return number;
    }

    /**
     * Convert a javascript date to SQL date (YYYY-MM-DD).
     * 
     * @param Date date
     *   Date to convert.
     * @returns string
     */
	function toSQLString(date) {
		return date.getFullYear() + '-' + pad(date.getMonth() + 1) + '-' + pad(date.getDate());
	}
	
	/**
	 * Convert a javascript date to DD-MM.
     * 
     * @param Date date
     *   Date to convert.
     * @returns string
	 */
	function toDateString(date) {
		return pad(date.getDate()) + '-' + pad(date.getMonth() + 1);
	}

	/**
	 * Convert a javascript date to day abbreviation (3 letters).
     * 
     * @param Date date
     *   Date to convert.
     * @returns string
	 */
	function toWeekDay(date) {
		return ['Zon', 'Maa', 'Din', 'Woe', 'Don', 'Vrij', 'Zat'][date.getDay()];
	}

	/*
	 * Build the base table and fill it with buttons.
	 * 
	 * @param object data
	 *   General data about the entity we are working with (Lead or Deal).
	 * @param object details
	 *   Detailed data about the entity.
	 * @returns void
	 */
	function buildTable(data, details)
	{
		// Clear table.
		$('#planning table').html('<thead><tr><th>Tijd</th></tr></thead><tbody></tbody>');
		
		// All possible timeslots.
		var allTimeslots = ['09.00', '10.00', '11.00', '13.00', '14.00', '15.00', '16.00']; // Tuesday
		// All available time slots.
		// Number of people on intake schedule for every timeslots in the allTimeslots array..
		var maxIntakes = [
			[ 0,       0,       0,       0,       0,       0,       0     ], // Sunday
			[ 1,       1,       1,       1,       1,       1,       1     ], // Monday
			[ 1,       1,       1,       1,       1,       1,       1     ], // Tuesday
			[ 1,       1,       1,       1,       1,       1,       1     ], // Wednesday
			[ 1,       1,       1,       1,       1,       1,       1     ], // Thursday
			[ 1,       1,       1,       1,       1,       1,       1     ], // Friday
			[ 0,       0,       0,       0,       0,       0,       0     ], // Saturday
		];

		// Get the contact ID if this is a Deal.
		var klantID = '';
		if ( data.Entity == 'Deals' )
		{
			console.log(details);
			klantID = details.data[0].Contact_Name.id;
		}
		
		// Create the header row, 14 days ahead, skipping the days without timeslots.
		var headerDate = new Date();
		for( var d = 0; d <= 14; d++ )
		{
			// Only create header entries for the weekdays.
			dow = headerDate.getDay();
			// Check if there is a plannable slot on the day of the week.
			var plannable = false;
			for ( var i = 0; i < maxIntakes[dow].length; i++ )
			{
				if ( maxIntakes[dow][i] > 0 )
				{
					plannable = true;
					break;
				}
			}
			if ( plannable )
			{
				$('#planning thead tr').append('<th>' + toWeekDay(headerDate) + ' ' + toDateString(headerDate) + '</th>');
			}
			headerDate.setDate(headerDate.getDate() + 1);
		}

		// Create the time slot rows.
		for ( var n = 0; n < allTimeslots.length; n++ )
		{
			// CReate the row HTML.
			var row = '<tr><th>' + allTimeslots[n] + '</th>';

			// Create cells 14 days ahead, skipping the days without timeslots.
			var headerDate = new Date();
			for( var d = 0; d <= 14; d++ )
			{
				// Only create cells that are allowed according to the timeslots.
				var dow = headerDate.getDay();
				// Check if there is a plannable slot on the day of week.
				var plannable = false;
				for ( var i = 0; i < maxIntakes[dow].length; i++ )
				{
					if ( maxIntakes[dow][i] > 0 )
					{
						plannable = true;
						break;
					}
				}
				if ( plannable )
				{
					if ( maxIntakes[dow][n] > 0 )
					{
						// Show a button. The cell get's a constructed ID, so it can later be fillen with the calls already scheduled.
						row = row + '<td id="id-' + toDateString(headerDate) + '-' + allTimeslots[n].replace('.', '-') + '">' +
						    '<p>Max intakes: <span class="max-intakes">' + maxIntakes[dow][n] + '<span></p>' +
						    '<div class="cell-content"></div>' +
							'<button class="plan-call" ' +
								'data-time="' + allTimeslots[n].replace('.', ':')  + '" ' +	// HH:MM of the time slot.
								'data-date="' + toSQLString(headerDate) + '" ' +			// YYYY-MM-DD of the time slot.
								'data-entity="' + data.Entity + '" ' +						// Lead or Deal, depending on what page we are on..
								'data-entityid="' + data.EntityId[0] + '" ' +				// Lead or deal ID.
								'data-klantid="' + klantID + '">Inplannen</button>' +	// If this is a deal, get the contact ID. For leads, this is the same as the lead ID.
						'</td>';
					}
					else {
						// Time slot marked as XX.XX: unplannable.
						row = row + '<td class="unplannable"></td>';
					}
				}

				// Move to the next day.
				headerDate.setDate(headerDate.getDate() + 1);
			}
			
			// Finalize the row HTML and append it to the table.
			row = row + '</tr>';
			$('#planning tbody').append(row);
		}
		
		// Fetch all planned intake calls and place them in the table.
		fetchCalls(data);

	}

	/*
	 * Fetch all calls and place planned intake calls on table.
	 * 
	 * @param object data
	 *   General data about the entity we are working with (Lead or Deal).
	 * @returns void
	 */
	function fetchCalls(data)
	{
		var pageNr = 1;				// Result page number when querying Zoho for the planned activites.
		var more = true;			// Any more records to find?
		var today = new Date();		// Needed to check found calls. We only want calls for today and in the future.
		
		today = toSQLString(today);
		
		do
		{
			more = false; // Assume only one page of search search result.
			
			// Find (the next) 200 outbound intake calls.
			ZOHO.CRM.API.searchRecord({Entity:"Calls",Type:"criteria",Query:"((Call_Type:equals:Outbound)and(Call_Purpose:equals:Intakegesprek))",page:pageNr,per_page:200})
			.then(function(response)
			{
				// Did Zoho report more records?
				more = response.info.more_records;
				
				// Process all found calls.
				do
				{
					// Get the individual call from the array and remove it.
					let call = response.data.shift();
					
					// Check if the call is scheduled today or in the future.
					var startTime = call.Call_Start_Time.substr(0,10);
					if ( startTime.localeCompare(today) >= 0 )
					{
						// Generate the ID of the table cell that this call belongs in.
						var id = '#id-' + call.Call_Start_Time.substr(8,2) + '-' + call.Call_Start_Time.substr(5,2) + '-' + call.Call_Start_Time.substr(11,5).replace(':', '-');

						// Does the slot exist in the table? If not, we can skip this call (it's probably too far in the future).
						if ( $(id).length > 0 )
						{
							var maxIntakes = parseInt($(id + ' .max-intakes').html(), 10);
							var name = call.Owner.name;
							
							// Add the call owner name.
							var content = $(id + ' .cell-content').html();
							if ( content.length == 0 )
							{
								content = '<p>- ' + name + '</p>';
							}
							else {
								content = content + '<p>- ' + name + '</p>';
							}
							$(id + ' .cell-content').html(content);
							
							// Count the number of names. If it has reached the maximum intakes, remove the button.
							var names = $(id + ' .cell-content p').length;
							if ( names >= maxIntakes )
							{
								$(id + ' button').remove();
							}
						}
					}
				} while ( response.data.length > 0);
			});
			
			// On to the next page of search results.
			pageNr++;
			
		} while (more);
		
	}
	
	/*
	 * Subscribe to the EmbeddedApp onPageLoad event before initializing the widget 
	 */
	ZOHO.embeddedApp.on("PageLoad",function(data)
	{
		
	 	// Verify if EntityInformation is passed.
		if(data && data.Entity)
		{
			
			// Get the entity details and build the table.
			ZOHO.CRM.API.getRecord({'Entity':data.Entity,'RecordID':data.EntityId[0]}).then(function(details) {

				// Build the table.
				buildTable(data, details);
			});
			
		}

	});
	
	/*
	 * initialize the widget.
	 */
	ZOHO.embeddedApp.init();
	
	/*
	 * Button click handler.
	 * Create an intake call for Sophie and send an intake planning mail.
	 */
	$(document).on('click', '.plan-call', function() {
		clickedButton = $(this);
		var date = $(this).data('date');
		var time = $(this).data('time');
		var entity = $(this).data('entity');
		var entityId = $(this).data('entityid');
		var klantID = $(this).data('klantid');
		// For leads, the enity ID and the klant ID are the same. Do not add the Who_Id parameter.
		if ( entity == 'Leads' ) {
			var data = [{
				'Call_Start_Time': date+'T'+time+':00+01:00',
				'Call_Purpose': 'Intakegesprek',
				'Owner': '294376000000277001', // Sophie Verdaasdonk.
				'What_Id': entityId,
				'Subject': 'intake afspraak #widget',
				'Call_Type': 'Outbound',
				'$se_module': entity
			}];
		}
		else {
			var data = [{
				'Call_Start_Time': date+'T'+time+':00+01:00',
				'Call_Purpose': 'Intakegesprek',
				'Owner': '294376000000277001', // Sophie Verdaasdonk.
				'What_Id': entityId,
				'Who_Id': klantID,
				'Subject': 'intake afspraak #widget',
				'Call_Type': 'Outbound',
				'$se_module': entity
			}];
		}
		
		// Create the call.
		ZOHO.CRM.API.insertRecord({Entity:"Calls",APIData:data,Trigger:["workflow"]}).then(function(data){
			clickedButton.parent().find('.cell-content').append('<p>- Sophie Verdaasdonk</p>');
		},function(error){console.log(error);});
		
		// Add the intake date and intake moment to the Lead or Deal.
		// This will kick off a workflow to send the intake planning mail to the customer.
		if ( entity == 'Leads' )
		{
			var entityData = [{
				'id': entityId,
				'Intake_datum': date,
				'Intake_moment': 'rond ' + time.replace(':', '.') + ' uur'
			}];
		}
		else
		{
			var entityData = [{
				'id': entityId,
				'Intakedatum': date,
				'Intakemoment': 'rond ' + time.replace(':', '.') + ' uur'
			}];
		}
		ZOHO.CRM.API.updateRecord({Entity:entity,APIData:entityData,Trigger:["workflow"]});
	});
	
}
